#version 330
#define MAX_POWERUPS 10

uniform mat4 uni_ModelViewMatrix;
uniform mat4 uni_ProjMatrix;
uniform mat3 uni_NormalMatrix; //transforms normals/tangents to VIEW space
uniform vec3 uni_ParallelLightDirection; //already in VIEW space
uniform vec4 uni_TextureMatrix1[MAX_POWERUPS]; //projector projection + normalization matrix
uniform vec4 uni_TextureMatrix2[MAX_POWERUPS]; //projector projection + normalization matrix
uniform vec4 uni_TextureMatrix3[MAX_POWERUPS]; //projector projection + normalization matrix
uniform vec4 uni_TextureMatrix4[MAX_POWERUPS]; //projector projection + normalization matrix
uniform vec4 uni_ModelProjectorMatrix1[MAX_POWERUPS]; //projector view matrix * model matrix
uniform vec4 uni_ModelProjectorMatrix2[MAX_POWERUPS]; //projector view matrix * model matrix
uniform vec4 uni_ModelProjectorMatrix3[MAX_POWERUPS]; //projector view matrix * model matrix
uniform vec4 uni_ModelProjectorMatrix4[MAX_POWERUPS]; //projector view matrix * model matrix

uniform mat4 uni_ModelLightMatrix; //=light view matrix * model matrix
uniform mat4 uni_LightNormMatrix; //combines light projection and normalization matrix

in vec3 in_Vertex;
in vec3 in_Normal;
in vec2 in_Tex;
in vec3 in_Tan;
in vec3 in_BiTan;

out vec2 var_TexCoord;
//out vec3 var_Normal; //transformed normal in view space
out vec3 var_EyeVec; //from point to eye (in TANGENT space)
out vec3 var_LightVec; //from point to light (in TANGENT space)
out vec4 var_Pos;

out vec4 var_TexCoordProj[MAX_POWERUPS]; //projective texturing coordinates
out vec4 var_SMTexCoords; //shadow map texture coordinates

void main(void)
{
	//transform normals
	//var_Normal = normalize(uni_NormalMatrix * in_Normal);

	//normal in view coords
	vec3 vViewNormal = uni_NormalMatrix * in_Normal;
	vec3 vViewTangent = uni_NormalMatrix * in_Tan;
	vec3 vViewBitangent = uni_NormalMatrix * in_BiTan;

	//calculate eye vector (= -pos vector in view space)
	//eye is at 0,0,0
	vec4 vPos4 = vec4(in_Vertex,1);
	vec4 vPosViewCoords = uni_ModelViewMatrix * vPos4;
	vec3 vViewEyeVec = -vec3(vPosViewCoords);
	//transform to tangent space
	var_EyeVec.x = dot(vViewEyeVec,vViewTangent);
	var_EyeVec.y = dot(vViewEyeVec,vViewBitangent);
	var_EyeVec.z = dot(vViewEyeVec,vViewNormal);

	//dont need to calculate light vector because we have a parallel light
	//transform light vec to tangent space
	var_LightVec.x = dot(-uni_ParallelLightDirection,vViewTangent);
	var_LightVec.y = dot(-uni_ParallelLightDirection,vViewBitangent);
	var_LightVec.z = dot(-uni_ParallelLightDirection,vViewNormal);

	// interpolate tex coords, simple interpolation
	var_TexCoord = in_Tex;

	var_Pos = uni_ProjMatrix * vPosViewCoords;
	//final vtx position - reuse pos from eye calc
	gl_Position =  var_Pos;
	
	//texture coords for projective texturing
	mat4 uni_TextureMatrix[MAX_POWERUPS];
	mat4 uni_ModelProjectorMatrix[MAX_POWERUPS];

    for(int i=0;i<MAX_POWERUPS;i++) 
	{		
		uni_TextureMatrix[i] = mat4(uni_TextureMatrix1[i],uni_TextureMatrix2[i],uni_TextureMatrix3[i],uni_TextureMatrix4[i]);
		uni_ModelProjectorMatrix[i] = mat4(uni_ModelProjectorMatrix1[i],uni_ModelProjectorMatrix2[i],uni_ModelProjectorMatrix3[i],uni_ModelProjectorMatrix4[i]);
		var_TexCoordProj[i] = uni_TextureMatrix[i] * uni_ModelProjectorMatrix[i] * vPos4;
	}

	//transform vtx to light coords
	var_SMTexCoords = uni_LightNormMatrix * uni_ModelLightMatrix * vPos4;
}